"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeneratorCore = void 0;
const change_case_1 = require("change-case");
const fs_extra_1 = __importDefault(require("fs-extra"));
const n_readlines_1 = __importDefault(require("n-readlines"));
const path_1 = __importDefault(require("path"));
require("reflect-metadata");
const type_graphql_1 = require("type-graphql");
class GeneratorCore {
    static #indexPath = path_1.default.resolve(__dirname, `./src/models/generated-joi-schemas/index.ts`);
    static #writeToPath = '';
    static #arrayOfComponent = [];
    static emitSchemaFile(arrayOfResolver, removeOperation, replaceInput, entity, fn) {
        this.#writeToPath = path_1.default.resolve(__dirname, `./prisma/generated/generated-${(0, change_case_1.paramCase)(entity)}-schema.graphql`);
        let resolverToBuild = [];
        if (arrayOfResolver.length === 0) {
            this.#arrayOfComponent.forEach((component) => {
                const target = [`FindMany${entity}Resolver`, `FindUnique${entity}Resolver`, `UpdateOne${entity}Resolver`, `CreateOne${entity}Resolver`, `DeleteOne${entity}Resolver`];
                if (target.includes(component.name)) {
                    resolverToBuild.push(component);
                }
            });
        }
        else {
            resolverToBuild = arrayOfResolver;
        }
        (0, type_graphql_1.buildSchemaSync)({
            resolvers: resolverToBuild,
            emitSchemaFile: this.#writeToPath,
            validate: false
        });
        let typeDefs = fs_extra_1.default.readFileSync(this.#writeToPath, 'utf8');
        if (removeOperation)
            typeDefs = typeDefs.replace(/type Mutation\W(\s*?){(.*?)}/gs, '').replace(/type Query\W(\s*?){(.*?)}/gs, '');
        if (replaceInput)
            typeDefs = typeDefs.replaceAll('CreateInput', 'AddInput').replaceAll('UpdateInput', 'EditInput');
        this.#objectJoiSchema(`${entity}AddInput`, typeDefs);
        this.#objectJoiSchema(`${entity}EditInput`, typeDefs);
        this.#objectJoiSchema(`${entity}WhereUniqueInput`, typeDefs);
        this.#enumJoiSchema(`${entity}ScalarFieldEnum`, typeDefs);
        if (fn)
            typeDefs = fn(typeDefs);
        fs_extra_1.default.writeFileSync(this.#writeToPath, typeDefs);
    }
    static #getListOfEnum(enumRegex, typeDefs) {
        let m;
        // const enumRegex = /enum (.*?) {/gs
        const allEnum = [];
        while ((m = enumRegex.exec(typeDefs)) !== null) {
            // This is necessary to avoid infinite loops with zero-width matches
            if (m.index === enumRegex.lastIndex) {
                enumRegex.lastIndex++;
            }
            // The result can be accessed through the `m`-variable.
            // m.forEach((match, groupIndex) => {
            //     console.log(`Found match, group ${groupIndex}: ${match}`)
            // })
            allEnum.push(m[1]);
        }
        return allEnum;
    }
    static #findWhatIWant(regex, typeDefs) {
        let mm1;
        let a = '';
        while ((mm1 = regex.exec(typeDefs)) !== null) {
            // This is necessary to avoid infinite loops with zero-width matches
            if (mm1.index === regex.lastIndex) {
                regex.lastIndex++;
            }
            a = mm1[0];
        }
        return a;
    }
    static #defaultHandler(allEnum, typeDefs, DSL, thingsToWrite) {
        let found = false;
        allEnum.forEach((x) => {
            const isRequired = DSL[1].trim().indexOf('!') > 0;
            const y = DSL[1].trim().replace('!', '');
            if (y.trim() === x.trim()) {
                let mm;
                const enumRegex = new RegExp('enum ' + y + ' {(.*?)}', 'gs');
                let allEnumFields = [];
                while ((mm = enumRegex.exec(typeDefs)) !== null) {
                    // This is necessary to avoid infinite loops with zero-width matches
                    if (mm.index === enumRegex.lastIndex) {
                        enumRegex.lastIndex++;
                    }
                    // The result can be accessed through the `m`-variable.
                    // mm.forEach((match, groupIndex) => {
                    //     console.log(`Found match, group ${groupIndex}: ${match}`)
                    // })
                    allEnumFields = mm[1]
                        .replace(/(?:\\[rn]|[\r\n]+)+/g, '')
                        .trim()
                        .split('  ');
                }
                thingsToWrite += `${DSL[0]}: Joi.string().valid(...${JSON.stringify(allEnumFields)})${isRequired ? '.required()' : ''},\n`;
                found = true;
            }
        });
        if (!found)
            thingsToWrite += `${DSL[0]}: Joi.any(),\n`;
        return thingsToWrite;
    }
    static #objectJoiSchema(stringToFind, typeDefs) {
        const enumRegex = /enum (.*?) {/gs;
        const allEnum = this.#getListOfEnum(enumRegex, typeDefs);
        const regex = RegExp(`input ${stringToFind}\\W?(s*?){(.*?)}`, 'gs');
        const a = this.#findWhatIWant(regex, typeDefs);
        if (a) {
            const cachePath = path_1.default.resolve(__dirname, './prisma/generated/generated-cache.json');
            const tmp = a.toString();
            fs_extra_1.default.writeFileSync(cachePath, tmp);
            const liner = new n_readlines_1.default(cachePath);
            let line;
            let thingsToWrite = '';
            while ((line = liner.next())) {
                const tempLine = line.toString('ascii');
                if (tempLine.indexOf('{') >= 0) {
                    thingsToWrite += `import { Joi } from 'cm-libs-bootstrap'\n`;
                    thingsToWrite += `export const ${stringToFind}JoiSchema = {\n`;
                }
                else if (tempLine.indexOf(':') <= 0)
                    thingsToWrite += `${tempLine}\n`;
                else {
                    const DSL = tempLine.split(': ');
                    switch (DSL[1].trim()) {
                        case 'DateTime!': {
                            thingsToWrite += `${DSL[0]}: Joi.date().required(),\n`;
                            break;
                        }
                        case 'DateTime': {
                            thingsToWrite += `${DSL[0]}: Joi.date(),\n`;
                            break;
                        }
                        case 'String!': {
                            thingsToWrite += `${DSL[0]}: Joi.string().required(),\n`;
                            break;
                        }
                        case 'String': {
                            thingsToWrite += `${DSL[0]}: Joi.string(),\n`;
                            break;
                        }
                        case 'Decimal!':
                        case 'Int!': {
                            thingsToWrite += `${DSL[0]}: Joi.number().required(),\n`;
                            break;
                        }
                        case 'Decimal':
                        case 'Int': {
                            thingsToWrite += `${DSL[0]}: Joi.number(),\n`;
                            break;
                        }
                        default: {
                            thingsToWrite = this.#defaultHandler(allEnum, typeDefs, DSL, thingsToWrite);
                        }
                    }
                }
            }
            const outputPath = path_1.default.resolve(__dirname, `./src/models/generated-joi-schemas/pc-${stringToFind}.schema.ts`);
            fs_extra_1.default.outputFileSync(outputPath, thingsToWrite);
            fs_extra_1.default.appendFileSync(this.#indexPath, `export * from './pc-${stringToFind}.schema'\n`);
            fs_extra_1.default.removeSync(cachePath);
        }
    }
    static #enumJoiSchema(stringToFind, typeDefs) {
        let thingsToWrite = '';
        thingsToWrite += `import { Joi } from 'cm-libs-bootstrap'\n`;
        thingsToWrite += `export const ${stringToFind}JoiSchema = Joi.string().valid(\n`;
        let mm;
        const enumRegex = new RegExp('enum ' + stringToFind + ' {(.*?)}', 'gs');
        let allEnumFields = [];
        while ((mm = enumRegex.exec(typeDefs)) !== null) {
            // This is necessary to avoid infinite loops with zero-width matches
            if (mm.index === enumRegex.lastIndex) {
                enumRegex.lastIndex++;
            }
            // The result can be accessed through the `m`-variable.
            // mm.forEach((match, groupIndex) => {
            //     console.log(`Found match, group ${groupIndex}: ${match}`)
            // })
            allEnumFields = mm[1]
                .replace(/(?:\\[rn]|[\r\n]+)+/g, '')
                .trim()
                .split('  ');
        }
        thingsToWrite += `...${JSON.stringify(allEnumFields)}\n)\n`;
        const outputPath = path_1.default.resolve(__dirname, `./src/models/generated-joi-schemas/pc-${stringToFind}.schema.ts`);
        fs_extra_1.default.outputFileSync(outputPath, thingsToWrite);
    }
    static startUp(allComponents) {
        fs_extra_1.default.removeSync(this.#indexPath);
        let thingsToWrite = '';
        thingsToWrite += `import { Joi } from 'cm-libs-bootstrap'\n`;
        thingsToWrite += `export const SortOrderSchema = Joi.string().valid(...['asc', 'desc'])\n`;
        const outputPath = path_1.default.resolve(__dirname, `./src/models/generated-joi-schemas/pc-SortOrderSchema.schema.ts`);
        fs_extra_1.default.outputFileSync(outputPath, thingsToWrite);
        const functions = Object.values(allComponents);
        functions.forEach((component) => {
            if (component.constructor.name === 'Function') {
                this.#arrayOfComponent.push(component);
            }
        });
    }
}
exports.GeneratorCore = GeneratorCore;
//# sourceMappingURL=generator.js.map